#pragma once

#include "string_view.h"
#include "Owned_string.h"

#include <cstddef>
#include <compression.h>

class Shared_string
{
public:
    Shared_string();

    Shared_string(Shared_string const&);
    Shared_string(Shared_string&&) noexcept;
    Shared_string(string_view);
    Shared_string(Owned_string&&);

    ~Shared_string();

    Shared_string& operator=(Shared_string const&);
    Shared_string& operator=(Shared_string&&) noexcept;
    Shared_string& operator=(string_view);
    Shared_string& operator=(Owned_string&&);

    char operator[](size_t) const;

    Shared_string substring(size_t start) const;
    Shared_string substring(size_t start, size_t size) const;

    size_t size() const;
    bool empty() const;

    char const* begin() const;
    char const* end() const;

    char const* c_str() const;

    operator string_view() const;
    operator Owned_string() &&;

    size_t ref_count() const;

private:
    void decrement_();
    void increment_();
    void pod_assign_(Shared_string const&);

    size_t*     ref_count_;
    char*       data_;
    string_view view_;
};
